/*
* TIDA-00386 FIrmware code for IR Remote
*
* This code is for MSP430 device in TIDA-00386 design. MSP430 device
* decodes the remote inputs and generate PWM duty cycle command  at 1.5 kHz
* for fan speed control.
*
* Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

#include <msp430g2201.h>

unsigned long Switch, Last_Switch, Prev_Switch, bigcommand;
unsigned long mask = 0x0000FF00;
unsigned char Code_sent_cnt;
unsigned char Remote_array_code[11]={0,0,0,0,0,0,0,0,13,10};
unsigned char High_cnt,Low_cnt, Int_cnt=0;
unsigned char command = 0;

unsigned char Output_Period = 0, Set_Output =6;

#define BTL		450
#define BTH		650

#define UARTENABLE

unsigned char Flag;				//this is a big flag which checks about the below
#define Switch_pressed			BIT0
#define Start_remote_process	BIT1
#define	Start_bit_flag			BIT2
#define Resend_bit_flag			BIT3
#define valid_start_high		BIT4


#define UP_Key			0x00FF58A7             // Increment Key-code
#define DOWN_Key		0x00FF6897             // Decrement Key-code
#define DIR_Key			0x00FF3AC5

void Init_clock(void)
{
    BCSCTL2 = SELM_0 + DIVM_0 + DIVS_0;

    if (CALBC1_1MHZ != 0xFF) 
    {
        /* Follow recommended flow. First, clear all DCOx and MODx bits. Then
         * apply new RSELx values. Finally, apply new DCOx and MODx bit values.
         */
        DCOCTL = 0x00;
        BCSCTL1 = CALBC1_1MHZ;      /* Set DCO to 1MHz */
        DCOCTL = CALDCO_1MHZ;
    }

    BCSCTL1 |= XT2OFF + DIVA_0;
    BCSCTL3 = XT2S_0 + LFXT1S_2 + XCAP_1;
}

void Init_timer1(void)
{
 
 	TAR=0;
    TACCTL0 = CM_0 + CCIS_0 + OUTMOD_0+CCIE ;

    /* TA1CCR0, Timer_A Capture/Compare Register 0 */
    TACCR0 = 12000;

    /* 
     * TA1CTL, Timer_A3 Control Register
     * 
     * TASSEL_2 -- SMCLK
     * ID_0 -- Divider - /1
     * MC_1 -- Up Mode
     */
    TACTL = TASSEL_2 + ID_0 + MC_1;
}

void Process_Remote_Code(void)
{
	if (Flag&Switch_pressed)			//Switch_pressed is like an interrupt flag
  	{
  		Flag&=~(Switch_pressed);		//Disable interrupt flag like you would disable a flag in an ISR
  		Last_Switch=Switch;				//Store Switch( I think this is the 32 bit data obtained) in Last Switch and Prev Switch

  		Flag|=(Start_remote_process);	//Set the Start Remote Process flag
        if (Last_Switch==UP_Key)
        {
            Set_Output += 1;
            if (Set_Output > 6) Set_Output = 6;
        }
        else if (Last_Switch==DOWN_Key)
        {
            Set_Output -= 1;
            if (Set_Output < 3) Set_Output =3;
        }
        else if (Last_Switch==DIR_Key)
        {
        	P1OUT ^= BIT6;					// Direction Change
        }
  	}	
	
}

void main(void)
{
	WDTCTL = WDTPW + WDTHOLD;                // Stop watchdog timer
	Init_clock();
  
	P1DIR |= BIT7;
	P1OUT |= BIT7;							// PWM Output

	P1DIR |= BIT6;							// Direction Pin
	P1OUT &= ~BIT6;							// Forward direction at start

	///////////////////P1.3 pulled high and configured for Hi/lo interrupt edge//////////
	P1REN|=BIT3;							//Push button pulled high
  	P1OUT|=BIT3;
	P1DIR &= ~BIT3;
	P1IE |= BIT3;                             // P2.6 interrupt enabled
	P1IES |= BIT3;                            // P2.6 Hi/lo edge
	P1IFG &= ~BIT3;                           // P2.6 IFG cleared
	/////////////////////////////////////////////////////////////////////////////////////////////


  	Flag|=(Start_remote_process);				//This was set in the routine also

    /*WDT Timer Configuration*/
    WDTCTL      = WDT_MDLY_0_064;
    IE1         |= WDTIE;                 // enable WDT interrupt
    _EINT();                              // Enable interrupts

  	__bis_SR_register(GIE);

	 while (1)
	 {
		 Process_Remote_Code();
  	 }
}


#pragma vector=PORT1_VECTOR
__interrupt void Port_1(void)
{
		P1IFG&=~BIT3;
		P1IES^=BIT3;
		P1IE|=BIT3;
		Int_cnt++;
		if (Flag&Start_remote_process)
		{
			TACTL = TASSEL_2 + ID_0 + MC_0;
			TAR=0;
			Init_timer1();
			Flag&=~(Start_remote_process);
		}	
		else
		{
			if (Int_cnt%2!=0)
				{
				if (Flag&Start_bit_flag)
					{
					if (TAR>3*BTL && TAR<3*BTH)
						{
						Switch=Switch<<1;
						Switch|=0x01;
						}	
					else
						{
							Switch=Switch<<1;
					
						}		
					}
			
				else if (Flag & valid_start_high)
					{
					if (TAR>4*BTL && TAR<4*BTH)
					Flag|=Resend_bit_flag;
					else if (TAR>8*BTL && TAR<8*BTH)
					Flag|=Start_bit_flag;	
					}	
				}
			else
				{
				if (Flag & Resend_bit_flag)
					{
					if (TAR>BTL && TAR<BTH)
						{
							Flag&=~(Start_bit_flag+Resend_bit_flag+valid_start_high);
							Flag|=Switch_pressed;
							Switch=0x01;
						}	
					}	
				else if (TAR>16*BTL && TAR<16*BTH)
					Flag|=valid_start_high;
			
			
				}
			TACTL = TASSEL_2 + ID_0 + MC_0;
			TAR=0;
			TACTL = TASSEL_2 + ID_0 + MC_1;
		}	
	
	
	
}			


#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER1_A0_ISR_HOOK(void)
{
	Flag&=~(Start_bit_flag+Resend_bit_flag+valid_start_high);
	if (Int_cnt==68)
	Flag|=Switch_pressed;
	Int_cnt=0;
	TACCTL0 &=~(CCIE);
	Flag|=(Start_remote_process);
  		
		
}


#pragma vector=WDT_VECTOR
__interrupt void watchdog_timer(void)
{
    Output_Period += 1; if (Output_Period > 6) Output_Period = 0;

    if (Set_Output <= Output_Period)
    	P1OUT &= ~BIT7;
    else
    	P1OUT |= BIT7;
}

